//
//  PostNoteViewController.m
//  FrontEndTest
//
//

#import "AppState.h"
#import "Constants.h"
#import "PostNoteViewController.h"
#import "AFNetworking.h"

@import Firebase;

@interface PostNoteViewController ()<UINavigationControllerDelegate,UITextFieldDelegate>{

}

@property (strong,nonatomic) NSString * hostname;
@property (strong,nonatomic) NSString *idToken;
@property (weak, nonatomic) IBOutlet UITextView *noteText;
@property (weak, nonatomic) IBOutlet UITextView *messagesLoaded;

@end
@implementation PostNoteViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    
}



- (void)viewDidAppear:(BOOL)animated {
    //Option 1 - use your ip address if you have deployed your app on your local machine
    // _hostname  =@"http://YOUR-IP_ADDRESS:8080";
    //Option 2 - use the Google App Server's base if you have deployed your app on GAE
    // _hostname  =@"my-project-id.appspot.com";
    
    FIRUser *currentUser = [FIRAuth auth].currentUser;
    [currentUser getTokenForcingRefresh:YES
                             completion:^(NSString *_Nullable idToken,
                                          NSError *_Nullable error) {
                                 if (error) {
                                     // Handle error
                                     return;
                                 }
                                 
                                 // Store the idToken  in your app to be sent to your backend via HTTPS
                                 _idToken = idToken;
                                 [self formAfGetRequestWithToken:_idToken];
                                }];


}

- (IBAction)postNoteButton:(id)sender {
    
    NSString *note = _noteText.text;
    NSDictionary *body = @{@"message": note};
    NSString *endPoint = @"/notes";
    [self formAfPostRequestWithToken:_idToken body:body endPoint:endPoint completionBlock:^{[self formAfGetRequestWithToken:_idToken];  }];
    
    
}
-(void) formAfPostRequestWithToken:(NSString*)token body:(NSDictionary *)body endPoint:(NSString *)endPoint completionBlock:(void(^)(void))callbackBlock
{
  
    NSError *error;
    NSData *jsonData = [NSJSONSerialization dataWithJSONObject:body options:0 error:&error];
    NSString *jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
    
    AFURLSessionManager *manager = [[AFURLSessionManager alloc] initWithSessionConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration]];
    
    NSMutableURLRequest *req = [[AFJSONRequestSerializer serializer] requestWithMethod:@"POST" URLString:[NSString stringWithFormat:@"%@%@",_hostname,endPoint] parameters:nil error:nil];
    
    req.timeoutInterval= [[[NSUserDefaults standardUserDefaults] valueForKey:@"timeoutInterval"] longValue];
    [req setValue:@"application/json" forHTTPHeaderField:@"Content-Type"];
    
    if ([req.allHTTPHeaderFields objectForKey:@"Authorization"] == nil) {
        NSString *authValue = [NSString stringWithFormat:@"Bearer %@", token];
        [req setValue:authValue forHTTPHeaderField:@"Authorization"];
    }
    [req setHTTPBody:[jsonString dataUsingEncoding:NSUTF8StringEncoding]];
    
    manager.responseSerializer = [AFHTTPResponseSerializer serializer];
    
    [[manager dataTaskWithRequest:req completionHandler:^(NSURLResponse * _Nonnull response, id  _Nullable responseObject, NSError * _Nullable error) {
        
        if (!error) {
            callbackBlock();
            // NSLog(@"Reply JSON: %@", responseObject);
            if ([responseObject isKindOfClass:[NSDictionary class]]) {
                
            }
        } else {
            NSLog(@"Error: %@, %@, %@", error, response, responseObject);
        }
    }] resume];

}

-(void) formAfGetRequestWithToken:(NSString*) token
{

    AFURLSessionManager *manager = [[AFURLSessionManager alloc] initWithSessionConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration]];
    
    NSMutableURLRequest *req = [[AFJSONRequestSerializer serializer] requestWithMethod:@"GET" URLString:[NSString stringWithFormat:@"%@/notes",_hostname] parameters:nil error:nil];
    
    req.timeoutInterval= [[[NSUserDefaults standardUserDefaults] valueForKey:@"timeoutInterval"] longValue];
    [req setValue:@"application/json" forHTTPHeaderField:@"Content-Type"];
    
    if ([req.allHTTPHeaderFields objectForKey:@"Authorization"] == nil) {
        //NSLog(@"token is %@",token);
        NSString *authValue = [NSString stringWithFormat:@"Bearer %@", token];
        [req setValue:authValue forHTTPHeaderField:@"Authorization"];
    }
    
    
    [[manager dataTaskWithRequest:req completionHandler:^(NSURLResponse * _Nonnull response, id  _Nullable responseObject, NSError * _Nullable error) {
         if (!error) {
           // NSLog(@"Reply JSON: %@", responseObject);
            NSDictionary *responseDictionary = responseObject;
            
            if ([responseDictionary isKindOfClass:[NSDictionary class]]) {
                //set the textview to the messages
                //[_messagesLoaded setText:@"YAY!"];
            }
            else{
                NSMutableString * msgs = [NSMutableString stringWithCapacity:200];
                for (NSDictionary *item in responseDictionary)
                {
                   // [msgs appendString:@'\n'];
                    [msgs appendString:[item objectForKey:@"message"]];
                    [msgs appendString:@"\n"];
                }
                 [_messagesLoaded setText:msgs];
            }
        } else {
            NSLog(@"Error: %@, %@, %@", error, response, responseObject);
            
        }
    }] resume];
    
}

- (IBAction)signOutButton:(id)sender
{
    FIRAuth *firebaseAuth = [FIRAuth auth];
    NSError *signOutError;
    BOOL status = [firebaseAuth signOut:&signOutError];
    if (!status) {
        NSLog(@"Error signing out: %@", signOutError);
        return;
    }
    [AppState sharedInstance].signedIn = false;
    [self dismissViewControllerAnimated:YES completion:nil];
}


@end
